#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# RGBLedLightshow fr WS2812 RGB LED's
#
# Basiert auf dem Python "strandtest.py" Beispiel 
# Author: Tony DiCola (tony@tonydicola.com)
#
# Lizenz:
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# 
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# Bentigte Module werden importiert
import spidev 
import time 
import sys 
import signal

def sigterm_handler(signal, frame):
    colorWipe(strip, Color(0, 0, 0)) 
    sys.exit(0) 

signal.signal(signal.SIGTERM, sigterm_handler)


# SPI-Schnittstelle wird initiaslisiert
spi = spidev.SpiDev() 
spi.open(0,0) 

# Funktion um das Potentiometer ber den ADC MCP3008 auslesen zu knnen
def readadc(adcnum): 
	if adcnum >7 or adcnum <0: 
		return-1 
	r = spi.xfer2([1,8+adcnum <<4,0]) 
	adcout = ((r[1] &3) <<8)+r[2] 
	return adcout 


from neopixel import *

# RGB LED's werden konfiguriert
LED_COUNT      = 12      # Anzahl der LED's
LED_PIN        = 12      # GPIO-Pin an dem der RGB-Ledstrip angesteuert wird
LED_FREQ_HZ    = 800000  # LED-Signal Frequenz
LED_DMA        = 5       # DMA channel zur Signalerzeugung
LED_BRIGHTNESS = 255     # Helligkeitseinstellung (255 = Helligkeitsmaximum | 0 = Helligkeitsminimum)
LED_INVERT     = False   # Aktivierung der Invertiuerung durch Angabe von "True"


# Die jeweiligen Farbwechselfunktionen werden initialisiert
def colorWipe(strip, color, wait_ms=50):
	for i in range(strip.numPixels()):
		strip.setPixelColor(i, color)
		strip.show()
		time.sleep(wait_ms/1000.0)
   
def colorWipeWheel(strip, wait_ms=50):
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(255, 0, 0))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(0, 255, 0))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(0, 0, 255))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(100, 200, 50))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(255, 255, 255))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(80, 125, 170))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(125, 50, 125))
    strip.show()
    time.sleep(wait_ms/1000.0)
  for i in range(strip.numPixels()):
    value = int(readadc(0)/100)
    if value != 4:
      return
    strip.setPixelColor(i, Color(0, 0, 0))
    strip.show()
    time.sleep(wait_ms/1000.0)

def theaterChase(strip, breaknumber, color, wait_ms=50, iterations=10):
  for j in range(iterations):
    value = int(readadc(0)/100)
    if value != breaknumber:
      return
    for q in range(3):
      for i in range(0, strip.numPixels(), 3):
        strip.setPixelColor(i+q, color)
      strip.show()
      time.sleep(wait_ms/1000.0)
      for i in range(0, strip.numPixels(), 3):
        strip.setPixelColor(i+q, 0)

def wheel(pos):
	if pos < 85:
		return Color(pos * 3, 255 - pos * 3, 0)
	elif pos < 170:
		pos -= 85
		return Color(255 - pos * 3, 0, pos * 3)
	else:
		pos -= 170
		return Color(0, pos * 3, 255 - pos * 3)

def rainbow(strip, wait_ms=20, iterations=1):
  for j in range(256*iterations):
    value = int(readadc(0)/100)
    if value != 8:
      return
    for i in range(strip.numPixels()):
      strip.setPixelColor(i, wheel((i+j) & 255))
    strip.show()
    time.sleep(wait_ms/1000.0)

def rainbowCycle(strip, wait_ms=20, iterations=50):
  for j in range(256*iterations):
    value = int(readadc(0)/100)
    if value != 9:
      return
    for i in range(strip.numPixels()):
      strip.setPixelColor(i, wheel(((i * 256 / strip.numPixels()) + j) & 255))
    strip.show()
    time.sleep(wait_ms/1000.0)

def theaterChaseRainbow(strip, wait_ms=50):
  for j in range(256):
    for q in range(3):
      for i in range(0, strip.numPixels(), 3):
        value = int(readadc(0)/100)
        if value < 10:
          return
        strip.setPixelColor(i+q, wheel((i+j) % 255))
      strip.show()
      time.sleep(wait_ms/1000.0)
      for i in range(0, strip.numPixels(), 3):
        strip.setPixelColor(i+q, 0)


# Hauptprogramm:
# 
if __name__ == '__main__':
  strip = Adafruit_NeoPixel(LED_COUNT, LED_PIN, LED_FREQ_HZ, LED_DMA, LED_INVERT, LED_BRIGHTNESS)
  strip.begin()

  while True:
    
    # Potentiometerwert wird gemessen und ein entsprechender Wert fr die jeweilige erkannte Position vergeben
    value = round(readadc(0)/100,0)
    
    # Je nachdem welche Position gemessen wurde, wird nun das jeweilige Farbwechselprogramm gestartet
    
    if value == 0: 
        colorWipe(strip, Color(0, 0, 0)) # LED's Aus
    if value == 1:
        colorWipe(strip, Color(255, 0, 0))  # Rot 
    if value == 2:
        colorWipe(strip, Color(0, 255, 0))  # Blau
    if value == 3:
        colorWipe(strip, Color(0, 0, 255))  # Grn
    if value == 4:
        colorWipeWheel(strip)  
    if value == 5:
        theaterChase(strip, 5 , Color(127, 127, 127))  # weies Farbgewitter
    if value == 6:
        theaterChase(strip, 6 , Color(127,   0,   0))  # rotes Farbgewitter
    if value == 7:
        theaterChase(strip, 7 , Color(  0,   0, 127))  # blaues Farbgewitter
    if value == 8:
        rainbow(strip) # Regenbogenwechsel (alle)
    if value == 9:
        rainbowCycle(strip) # Regenbogenwechsel im Kreis
    if value > 9:
        theaterChaseRainbow(strip) # Regenbogen Farbgewitter
    
